import '../models/graph_edge.dart';
import '../models/graph_node.dart';
import 'package:flutter/material.dart';
import '../utils/node_arrangement.dart';
// * Equatable package messes up everything, do not use.

@immutable
abstract class GraphEvent {
  GraphEvent() : super();
}

class InitNodesEvent extends GraphEvent {
  final List<GraphNode> nodes;
  InitNodesEvent(this.nodes);
}

class RefreshEvent extends GraphEvent {
  final List<GraphNode> nodes;
  final List<GraphEdge> edges;
  RefreshEvent({this.nodes = const [], this.edges = const []});
}

class AddNodeEvent extends GraphEvent {
  final GraphNode node; // left-top position of the box
  AddNodeEvent({
    @required this.node, 
    });
}

class NodeArrangementEvent extends GraphEvent {
  final Offset center;
  final NodeArrange arrangementType;
  
  NodeArrangementEvent({
    @required this.center,
    @required this.arrangementType, 
    });
}

class RemoveNodeEvent extends GraphEvent {
  final String nodeID;
  RemoveNodeEvent(this.nodeID);
}

class ConnectNodesEvent extends GraphEvent {
  final String firstNodeID;
  final String secondNodeID;

  ConnectNodesEvent(this.firstNodeID, this.secondNodeID);
}

class DisconnectNodesEvent extends GraphEvent {
  final String firstNodeID;
  final String secondNodeID;

  DisconnectNodesEvent(this.firstNodeID, this.secondNodeID);
}
