import 'package:flutter/material.dart';
import 'package:relations/utils/math_util.dart';

class GraphNode {

  static const String INVALID_ID = "-1";
  static int _nextValidID = 1;
  static int get _getNextValidID => _nextValidID++;
  /// returns the String representation of the node
  static String get nextID => _getNextValidID.toString();

  /// should be a unique id, can be provided from outside. If omitted, this class will automatically generate one.
  String id;

  /// left-top position of the box
  Offset position; 

  /// size of the wrapperbox
  Size size;

  /// widget to be shown inside the wrapperbox
  Widget widget;

  /// simple visibility porperty (not used yet)
  bool visible;

  /// have fixed position
  bool fixed;

  /// holds any addational data
  dynamic data;         

  GraphNode(
  {
    String id,
    this.position = Offset.infinite,
    this.size = Size.zero,
    this.widget,
    this.visible = true,
    this.fixed = false,
    this.data
  }) {
    this.id = id ?? nextID;
  }

  GraphNode copy() {
    return GraphNode(
      id: this.id,
      position: this.position,
      size: this.size,
      widget: this.widget,
      visible: this.visible
    );
  }

  bool get positionUnset => this.position == null || this.position == Offset.infinite;

  @override bool operator ==(Object other) {
    return other is GraphNode && other.id == this.id;
  } 

  @override int get hashCode => hashValues(id, position, size, visible);

  @override String toString() {
    return 'id:$id pos:$position size:${size.toString()}';
  }

  Rect get boundingRect => Rect.fromLTRB(position.dx, position.dy, size.width, size.height);

  List<Offset> get globalBoundCoords  {
    return [
      leftTop,
      rightTop,
      rightBottom,
      leftBottom
    ];
  }

  void locateByCenter(Offset center) {
    this.position = Offset(center.dx - size.width / 2.0, center.dy - size.height / 2.0);
  }

  Offset get leftTop => position;
  Offset get rightTop => Offset(position.dx + size.width, position.dy);
  Offset get rightBottom => Offset(position.dx + size.width, position.dy + size.height);
  Offset get leftBottom => Offset(position.dx, position.dy + size.height);

  Offset get center => MathUtil.centreOfNode(this);
}
