import 'package:relations/relations.dart';
import 'package:relations/utils/collision.dart';
import '../bloc/bloc.dart';
import '../models/graph.dart';
import '../models/stage_config.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'stage.dart';

/// A simple wrapper stateful widget class which provides the neccessary blocs for the stage.
class StageWrapper extends StatefulWidget {
  final StageConfig stageConfig;

  StageWrapper({
    Key key,
    @required this.stageConfig
  }) : super(key: key);

  _StageWrapperState createState() => _StageWrapperState();
}

class _StageWrapperState extends State<StageWrapper> {

  StageBloc stageBloc;
  GraphBloc graphBloc;


  void _checkOverlap(List<GraphNode> nodes) {
    for (var i = 0; i < nodes.length; i++) {

      var nodeA = nodes[i];

      for (var j = 0; j < nodes.length; j++) {

        var nodeB = nodes[j];

        if (nodeA == nodeB) continue;

        Collision.checkAndResolve(nodeA, nodeB);
      }
    }
  }

  void sepatareNodes(List<GraphNode> nodes, {int iterations = 15}) {
    for (var i = 0; i < iterations; i++) {
      _checkOverlap(nodes);
    }   
  }

  void _placeUnpositioned(List<GraphNode> nodes) {

     nodes.forEach((GraphNode node) => {
       if (node.positionUnset) {
         node.locateByCenter(stageBloc.stageCenter)
       }
     });

     sepatareNodes(nodes);
  }

  @override void initState() {
    super.initState();

    stageBloc = StageBloc(stageSize: widget.stageConfig.stageSize, theme: widget.stageConfig.theme);

    _placeUnpositioned(widget.stageConfig.nodes);

    graphBloc = GraphBloc(graph: Graph.fromNodes(widget.stageConfig.nodes));
  }

  @override
  Widget build(BuildContext context) {
    
    return MultiBlocProvider(
      providers: [
        BlocProvider<StageBloc>(
          builder: (BuildContext context) => stageBloc
        ),
        BlocProvider<GraphBloc>(
          builder: (BuildContext context) => graphBloc
        )
      ],
      child: Stage(config: widget.stageConfig)
    );
  }

  @override 
  void dispose() {
    super.dispose();
    stageBloc.dispose();
    graphBloc.dispose();
  }
}
